using System;
using System.IO;
using System.Diagnostics;

/*
 * Class IntList represents a list of ints; it is our own version of List<int>. 
 * You create the list entries by using Add(), one at a time.
 * Once you have a list, you can print it (though I recommmend against this if the list has 1000 entries, let alone 100,000). 
 * 
 * You can get the nth value with get(n), or set the nth value with set(n, val).
 * If n is out of range, get(n) returns 0 and set(n, val) does nothing.
 * 
 */



class IntList {
    // instance variables - replace the example below with your own
    private int[] elements;
    private int currsize;

    /**
     * Constructor for objects of class IntList
     */
    public IntList()
    {
        // initialise instance variables
        const int defaultCapacity = 5;
        currsize = 0;
        elements = new int[defaultCapacity];
    }

    /**
     * parameter constructor, allowing you to predefine a capacity.
     */
    public IntList(int capacity) {
        currsize = 0;
        elements = new int[capacity];
    }
       
    /**
     * Add() Adds the value y to the list, at position currsize
     * (that is, at the end of the existing list).
     * The size of the list is incremented.
     * If pos = nums.length, there is no more room, so this version doesn't do anything;
     * THAT PART YOU ARE TO FIX!!
     */
    public void Add(int y) {
        if (currsize == elements.Length) {
            //expand();
        }
        elements[currsize] = y;
        currsize += 1;      // or currsize++
    }
    
    // y is the fill value
    public void Fill(int y) {
	for (int i=currsize; i<elements.Length; i++) {
		Add(y);
	}
    }

    public void RandomFill() {
	Random r = new Random();
	for (int i=currsize; i<elements.Length; i++) {
		Add(r.Next());
	}
    }

    // get nth value, with range check
    public int get(int n) {
        if (n<0 || n >= currsize) {
            Console.WriteLine("Warning: IntList.get() called with out-of-range n = " + n);
            return default(int);		// special thing for an otherwise annoying case!
        }
        return elements[n];
    }
    
    // set nth value, with range check
    public void set(int n, int val) {
        if (n<0 || n >= currsize) {
            Console.WriteLine("Warning: IntList.set() called with out-of-range n = " + n);
            return;
        }
        elements[n] = val;
    }
    
    public int size() {
        return currsize;
    }

    
     public void print2() {
         int i=0;
         while (i<currsize) {
            Console.WriteLine(i + ":  " + elements[i]);
            i++;    
        }
    }
    
    private void swap(int i, int j) {
	//if (i<0 || i >= currsize || j<0 || j>= currsize) return;
	if (i==j) return;
	int temp = elements[i];
	elements[i] = elements[j];
	elements[j] = temp;
    }

    // for merging equal-sized blocks
    private static void mergeblocks(int[] A, int[] T, int start, int blocksize) {
	mergeblocks2(A, T, start, blocksize, blocksize);
    }

    // recursive mergesort often involves unequal-sized blocks
    private static void mergeblocks2(int[] A, int[] T, int start, int block1, int block2) {
        // merge A[start]..A[start+block1-1] and A[start+block1]..A[end],
        // where end = min(start+block1+block2-1, len)
	// destination is temp[start...end]
	// note second block can be partial or empty
	int i = start;
        int j = start+block1;        // second block start
        
        int end1 = start+block1-1;
        int end2 = end1 +block2;

	if (end1 > A.Length-1) {	// no second block
	    for (i=start; i<=A.Length-1; i++) T[i] = A[i];
	    return;
	}
        
        if (end2 > A.Length-1) {
            end2 = A.Length-1;
        }
        
        int k = start;
        while (i<=end1 && j<=end2) {
            if (A[i]<A[j]) {
                T[k++] = A[i++];
            } else {
                T[k++] = A[j++];
            }
        }
        if (i>end1) { // copy rest of j
            while (k<=end2) T[k++] = A[j++];
        } else {
            while (k<=end2) T[k++] = A[i++];
        }
    }

    // merges consecutive blocks of size blocksize into sorted chunks of size 2*blocksize
    // This is more likely to be done with the nonrecursive mergesort.
    // This can also be done within imsort() rather than as a separate method.
    private static void mergepass(int [] A, int [] temp, int blocksize) {
	int bstart = 0;
	// ...
    }

    // nonrecursive mergesort
    // you can also do the recursive version
    public void imsort() {
   	Stopwatch s = new Stopwatch();
	s.Start();
	int[] temp = new int[elements.Length];		// create temporary space
	int blockLength = 1;
	while (blockLength < elements.Length) {
	    // do one pass, either inline or with mergepass
            // YOU NEED TO ADD STUFF HERE
	    //swap(elements, temp);
	    int[] a = elements;
	    elements = temp;
	    temp = a;
	    blockLength *= 2;		// double the blockLength each time
	}
	s.Stop();
	Console.WriteLine("mergesort took {0} milliseconds", s.ElapsedMilliseconds);
    }
    
    private static void swap(int[] a1, int[] a2) {
	int[] temp = a1;
	a1 = a2;
	a2 = temp;
    }

    // selection sort
    public void ssort() {
	Stopwatch s = new Stopwatch();
	s.Start();
	for (int i = 0; i<currsize-1; i++) {
	    // find smallest of elements[i]..elements[currsize-1] and swap to position i
	    int index_min = i;
	    int curr_min_val = elements[i];
	    for (int j = i+1; j<currsize; j++) {
		if (elements[j] < curr_min_val) {
		    curr_min_val = elements[j];
		    index_min = j;
		}
	    }
	    swap(i,index_min);
	}
	s.Stop();
	Console.WriteLine("selection sort took {0} milliseconds", s.ElapsedMilliseconds);
    }

    public void print() {
	for (int i = 0; i< elements.Length; i++) {
	    Console.Write(elements[i]);
	    Console.Write(' ');
	}
	Console.WriteLine();
    }

    public bool checksort() {
	for (int i=1; i<elements.Length; i++) {
	    if (elements[i-1] > elements[i]) return false;
	}
	return true;
    }
 }
