using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;

/**
 * Calculating Fib(n) with a stack instead of with recursion
 * There are 3 positions. When we first want to find fib(n), we push fibframe(1,n). 
 * If the top of the stack is a fibframe with pos=1, we push a fibframe(1,n-1)
 * If the top of the stack is a fibframe with pos=2, we push a fibframe(1,n-2)
 * If the top is a fibframe with pos=3, returning a value, we pop it and then increment 
 * the pos of the PREVIOUS stack frame (which cannot be 3). We also set the val_ for that frame
 */

class driver {

    private static bool debug = false;

    public static void Main(string[] args) {
	int N = 10;
        if (args.Length > 0) {
                N = Convert.ToInt32(args[0]);
                Console.WriteLine("N is {0}", N);
        }
	
	int res = fibr(N);
	Console.WriteLine("fib({0}) = {1}", N, res);	
	res = fibs(N);
	Console.WriteLine("stack method: fib({0}) = {1}", N, res);
    }

    public static int fibs(int n) {
	Stack<fibframe> S = new Stack<fibframe>();
	if (n<=1) return 1;
	S.Push(new fibframe(n,1));
	while (true) {
		fibframe ff = S.Get();
		n = ff.n();
		if (ff.pos() == 1) {
			if (n<=2) { // n-1 <= 1
			    S.Push(new fibframe(n-1,3,1));
			    if (debug) Console.WriteLine("pushing stub to return 1");
			} else {
			    S.Push(new fibframe(n-1,1));
			    if (debug) Console.WriteLine("pushing new call to F({0})", n-1);
			}
		} else if (ff.pos() == 2) {
			if (n<=3) {	// n-2 <= 1
			    S.Push(new fibframe(n-2,3,1));
			    if (debug) Console.WriteLine("pushing stub to return 1");
			} else {
			    S.Push(new fibframe(n-2,1));
			    if (debug) Console.WriteLine("pushing new call to F({0})", n-2);
			}
		} else { // ff.pos() == 3
			int result = ff.val();
			S.Pop();
			if (S.IsEmpty()) return result;
			ff = S.Get(); // previous frame; we just finished first or second subcall
			if (ff.pos() == 1) {
				ff.incpos();
				ff.setval(result);
				if (debug) Console.WriteLine("returning {0} to first subcall of F({1}) call", result, ff.n());
			} else if (ff.pos() == 2) {  // just found result of SECOND call
				ff.incpos();
				ff.setval(ff.val() + result);
				if (debug) Console.WriteLine("returning {0} to second subcall of F({1}) call; final result is {2}", result, ff.n(), ff.val());			}
		}
	}
	//return -1;
    }

    public static int fibr(int n) {
	if (n<=1) return 1;	/* 1 */
	int f1 = fibr(n-1);	/* 2 */
	int f2 = fibr(n-2);	/* 3 */
	return f1 + f2;
    }
	
}


/* 
 * Stack frame to represent where we are in calculating F(n).
 * pos_ represents one of the three positions: before F(n-1), between F(n-1) and F(n-2), and at the end.
 * If pos_ = 2, meaning the sub-value F(n-1) has been computed, then its value is stored in val_
 * If pos_ = 3, val_ represents the result of F(n-1) + F(n-2), that is, F(n).
 */
class fibframe {
	private int n_;
	private int pos_;	
	private int val_;	// Should be set to fib(n-1) if pos_==2, and to fib(n) if pos==3
	public fibframe(int n, int p) {n_ = n; pos_ = p; val_ = 0;}
	public fibframe(int n, int p, int v) {n_ = n; pos_ = p; val_ = v;}
	public int n() {return n_;}
	public int pos() {return pos_;}
	public void incpos() {pos_ += 1;}
	public int val() {return val_;}
	public void setval(int val) {val_ = val;}
}


class Stack<T> {

	private List<T> data_;
	
	public Stack() {
		data_ = new List<T>();
	}

	public void Push(T val) {
		data_.Add(val);
	}

	public T Pop() {
		T val = data_[data_.Count-1];
		data_.RemoveAt(data_.Count-1);
		return val;
	}
	
	public T Get() {
		T val = data_[data_.Count-1];
		return val;
	}
	
	public bool IsEmpty() {
		return data_.Count == 0;
	}
}


