using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;

/**
 * Simple program to create some linked lists.
 */

class driver {
    public static void Main(string[] args) {
	strlist sl = new strlist();
	//insert(sl);
	altinsert(sl);
	sl.print();
	Console.WriteLine("the size is {0}", sl.size());
	for (int n = sl.size()-1; n>=0; n--) {
		Console.WriteLine("string {0} is {1}", n, sl.Get(n));
	}

	search("zebra",sl);
	search("aardvark",sl);
	search("grass,",sl);
	search("eats",sl);
	search("food",sl);
	
	Console.WriteLine("iterator:");
	foreach (string s in sl.GetEnumerator2()) Console.WriteLine(s);
    }

    public static void search(string s, strlist t) {
	if (t.search(s)) Console.WriteLine("string \"{0}\" was found", s);
	else Console.WriteLine("string \"{0}\" was NOT THERE", s);
    }

    public static void insert(strlist sl) {
	sl.insert("zebra");
	sl.insert("fresh");
	sl.insert("antelope");
	sl.insert("grass,");
	sl.insert("with");
	sl.insert("eats");
	sl.insert("plays");
    }

    public static void altinsert(strlist sl) {
	sl.insert("grass,");
	sl.insert("eats");
	sl.insert("with");
	sl.insert("zebra");
	sl.insert("fresh");
	sl.insert("antelope");
	sl.insert("plays");
    }
}

// a simple class for string trees. Either rename to strlist, or embed this within strlist.

class strlist {
     
    // We do not provide a way to set the string in an existing node; that's a bad idea.
    class treenode {
	private string data_;
	private int count_;	// optional
	private treenode left_, right_;
	public treenode(string d, treenode l, treenode r) {
		data_ = d; count_ = 1; left_ = l; right_ = r;
	}
	public treenode(string d, int c, treenode l, treenode r) {
		data_ = d; count_ = c; left_ = l; right_ = r;
	}
	public string data() {return data_;}
	public int   count() {return count_;}
	public treenode left()  {return left_;}
	public treenode right() {return right_;}
	public void setcount(int count) {count_ = count;}
	public void setleft  (treenode t) {left_ = t;}
	public void setright (treenode t) {right_ = t;}
    }

    private treenode root_ = null;
    private int count_ = 0;

    public void inTraverse() {inTraverse(root_,0);}

    public void print() {inTraverse(root_, 0);}

    private void inTraverse(treenode p, int depth) {
	int indent = 0;
	if (p==null) return;
	inTraverse(p.left(), depth+1);
	for (int i = 0; i< depth*indent; i++) Console.Write(' ');
	Console.WriteLine(p.data());
	//if (p.left() == null && p.right() == null) Console.WriteLine("    leaf!");
	inTraverse(p.right(), depth+1);
    }

    // must not be called with val == null
    public void insert(string val) {
	if (root_ == null) {
		root_ = new treenode(val, null, null);
		count_ ++;
		return;
	} else {
		rinsert(val, root_);
	}
    }

    string getstr_ = null;

    public string Get(int n) {
	if (n>=count_) return null;
	getstr_ = null;
	get(n,root_);
	return getstr_;
    }

    // returns number of nodes it has visited, assuming this number is <=n, otherwise just >n.
    // if left subtree visits n nodes, then the current node is it.
    // as soon as we visit more than n nodes, we can quit.
    private int get(int n, treenode t) {
	if (t == null) return 0;
        //Console.WriteLine("get({0}) called on node {1}", n, t.data());
	int l = get(n,t.left());
	if (l>n) return l;	// found in left subtree
	if (l==n) {
		getstr_ = t.data();
		return n+1;
	}
	return l + 1+ get(n-l-1,t.right());
    }

    public int size() {return count_;}

    public bool search(string s) {return search(s, root_);}

    private bool search(string s, treenode t) {
	if (t==null) return false;
	if (t.data() == s) return true;
	if (search(s, t.left())) return true;
	return search(s, t.right());
    }

    // recursive insert: does not get called with t==null
    private void rinsert(string val, treenode t) {
	int comp = val.CompareTo(t.data()); // = 0 if strings ==, <0 if val<t.data, etc
	if (comp == 0 ) return;	// This is where you would update the count
	if (comp < 0) {		// val < t.data()
	    if (t.left() == null) {
		t.setleft(new treenode(val, null, null));
		count_++;
	    } else {
		rinsert(val, t.left());
	    }
	} else { // val > t.data()
	    if (t.right() == null) {
		t.setright(new treenode(val, null, null));
		count_++;
	    } else {
		rinsert(val, t.right());
	    }
	}
    }

    /* */
    // System.Collections.IEnumerable
    public IEnumerable GetEnumerator()  {
	foreach (string s in GetEnumerator(root_)) yield return s;
    }

    private static IEnumerable GetEnumerator(treenode p) {
	if (p==null) yield break;
	foreach (string s in GetEnumerator(p.left())) {yield return s;}
	yield return p.data();
	foreach (string s in GetEnumerator(p.right())) {yield return s;}
    }

    /* */


    // iterative traverse using a stack
    public IEnumerable GetEnumerator2() {
  	Stack<treenode> s = new Stack<treenode>();
	treenode n = root_;
	while (true) {
	    while (n!= null) {
		s.push(n);
		n = n.left();
	    }
	    if (s.isEmpty()) break;
	    n = s.pop();
	    yield return n.data();
	    n = n.right();
	} 
	//yield break;
    }

}


class Stack<T> {

	private List<T> data_;
	
	public Stack() {
		data_ = new List<T>();
	}

	public void push(T val) {
		data_.Add(val);
	}

	public T pop() {
		T val = data_[data_.Count-1];
		data_.RemoveAt(data_.Count-1);
		return val;
	}
	
	public bool isEmpty() {
		return data_.Count == 0;
	}
}

