using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;

/**
 * A collection of tree traversers
 */

class driver {
    public static void Main(string[] args) {
	strtree tree = new strtree();
	//insert(tree);
	altinsert(tree);
	Console.WriteLine("1. recursive traverser");
	tree.inTraverse();
	Console.WriteLine("1.9 Enumerator demo");
	foreach (int n in demoEnumerator(7)) Console.WriteLine(n);
	Console.WriteLine("2. recursive Enumerator");
	foreach (string s in tree.MyEnumerator()) Console.WriteLine(s);
	Console.WriteLine("3. iterative traverser using a stack");
	tree.inTraverseStack();
	Console.WriteLine("4. alternative iterative traverser using a stack");
	tree.inTraverseStackAlt();
	Console.WriteLine("5. queue traverser");
	tree.inTraverseQueue();
	Console.WriteLine("6. alternative queue traverser");
	tree.inTraverseQueueAlt();
	Console.WriteLine("7. stack-based Enumerator");
	foreach (string s in tree.MyEnumerator2()) Console.WriteLine(s);
	Console.WriteLine("8. NextSetup/Next traverser");
	tree.NextSetup();
	while (true) {
		string s = tree.Next();
		if (s==null) break;
		Console.WriteLine(s);
	}
    }

    public static void insert(strtree sl) {
	sl.insert("zebra");
	sl.insert("fresh");
	sl.insert("antelope");
	sl.insert("grass,");
	sl.insert("with");
	sl.insert("eats");
	sl.insert("plays");
    }

    public static void altinsert(strtree sl) {
	sl.insert("grass,");
	sl.insert("eats");
	sl.insert("with");
	sl.insert("zebra");
	sl.insert("fresh");
	sl.insert("antelope");
	sl.insert("plays");
    }

    public static IEnumerable demoEnumerator(int max){
	for (int i=1; i<=max; i++) yield return i*i;
    }
	
}

// a simple class for string trees. Either rename to strlist, or embed this within strlist.

class strtree {
     
    // We do not provide a way to set the string in an existing node; that's a bad idea.
    class treenode {
	private string data_;
	private int count_;	// optional
	private treenode left_, right_;
	public treenode(string d, treenode l, treenode r) {
		data_ = d; count_ = 1; left_ = l; right_ = r;
	}
	public treenode(string d, int c, treenode l, treenode r) {
		data_ = d; count_ = c; left_ = l; right_ = r;
	}
	public string data() {return data_;}
	public int   count() {return count_;}
	public treenode left()  {return left_;}
	public treenode right() {return right_;}
	public void setcount(int count) {count_ = count;}
	public void setleft  (treenode t) {left_ = t;}
	public void setright (treenode t) {right_ = t;}
    }

    private treenode root_ = null;

    // must not be called with val == null
    public void insert(string val) {
	if (root_ == null) {
		root_ = new treenode(val, null, null);
		return;
	} else {
		rinsert(val, root_);
	}
    }

    // recursive insert: does not get called with t==null
    private void rinsert(string val, treenode t) {
	int comp = val.CompareTo(t.data()); // = 0 if strings ==, <0 if val<t.data, etc
	if (comp == 0 ) return;	// This is where you would update the count
	if (comp < 0) {		// val < t.data()
	    if (t.left() == null) {
		t.setleft(new treenode(val, null, null));
	    } else {
		rinsert(val, t.left());
	    }
	} else { // val > t.data()
	    if (t.right() == null) {
		t.setright(new treenode(val, null, null));
	    } else {
		rinsert(val, t.right());
	    }
	}
    }

    // ========================================================

    // traditional recursive traversal, with depth parameter

    public void inTraverse() {inTraverse(root_, 0);}

    private void inTraverse(treenode p, int depth) {
	int indent = 4;
	if (p==null) return;
	inTraverse(p.left(), depth+1);
	for (int i = 0; i< depth*indent; i++) Console.Write(' ');
	Console.WriteLine(p.data());
	inTraverse(p.right(), depth+1);
    }


    // traverse using an iterator (enumerator)
    // System.Collections.IEnumerable
    public IEnumerable MyEnumerator()  {
	foreach (string s in MyEnumerator(root_)) yield return s;
    }

    private static IEnumerable MyEnumerator(treenode p) {
	if (p==null) yield break;
	foreach (string s in MyEnumerator(p.left())) {yield return s;}
	yield return p.data();
	foreach (string s in MyEnumerator(p.right())) {yield return s;}
    }


    // iterative traverse using a stack, simplified algorithm
    public void inTraverseStack() {
  	Stack<treenode> s = new Stack<treenode>();
	treenode n = root_;
	while (true) {
	    while (n!= null) {
		s.push(n);
		n = n.left();
	    }
	    if (s.isEmpty()) break;
	    n = s.pop();
	    Console.WriteLine(n.data());
	    n = n.right();
	} 
	
    }

    // iterative traverse using a stack of Objects, general algorithm
    public void inTraverseStackAlt() {
  	Stack<Object> s = new Stack<Object>();
    	s.push(root_);
    	while (!s.isEmpty()) {
            Object x = s.pop();
            if (x is treenode) {
                treenode t = x as treenode;
                if (t.right() != null) s.push(t.right());
                s.push(t.data());
                if (t.left() != null) s.push(t.left());
            } else {
                string str = x as string;
                Console.WriteLine(str);
            }
        }
    }

    // iterative traverse using a queue
    // what does this do?
    public void inTraverseQueue() {
  	Queue<treenode> s = new Queue<treenode>();
	treenode n = root_;
	while (true) {
	    while (n!= null) {
		s.push(n);
		n = n.left();
	    }
	    if (s.isEmpty()) break;
	    n = s.pop();
	    Console.WriteLine(n.data());
	    n = n.right();
	} 
	
    }


    // iterative traverse using a queue of Objects
    public void inTraverseQueueAlt() {
  	Queue<Object> s = new Queue<Object>();
    	s.push(root_);
    	while (!s.isEmpty()) {
            Object x = s.pop();
            if (x is treenode) {
                treenode t = x as treenode;
                if (t.left() != null) s.push(t.left());
                s.push(t.data());
                if (t.right() != null) s.push(t.right());
            } else {
                string str = x as string;
                Console.WriteLine(str);
            }
        }
    }



    // iterator using a stack
    public IEnumerable MyEnumerator2() {
  	Stack<treenode> s = new Stack<treenode>();
	treenode n = root_;
	while (true) {
	    while (n!= null) {
		s.push(n);
		n = n.left();
	    }
	    if (s.isEmpty()) break;
	    n = s.pop();
	    yield return n.data();
	    n = n.right();
	} 
    }


    // next is a way of iteration using NextSetup() and Next()

    private Stack<treenode> stack;
    private treenode current;
    public void NextSetup() {
	stack = new Stack<treenode>();
	current = root_;
	/*
	while (current != null) {
		stack.push(current);
		current = current.left();
	}
	/* */
    }

    public string Next() {
	while (current != null) {
		stack.push(current);
		current = current.left();
	}
	if (stack.isEmpty()) return null;
	current = stack.pop();
	string theString = current.data();
	current = current.right();
	return theString;
    }
	
}


class Stack<T> {

	private List<T> data_;
	
	public Stack() {
		data_ = new List<T>();
	}

	public void push(T val) {
		data_.Add(val);
	}

	public T pop() {
		T val = data_[data_.Count-1];
		data_.RemoveAt(data_.Count-1);
		return val;
	}

	public bool isEmpty() {
		return data_.Count == 0;
	}
}

class Queue<T> {

	private List<T> data_;
	
	public Queue() {
		data_ = new List<T>();
	}

	public void push(T val) {
		data_.Add(val);
	}

	public T pop() {
		T val = data_[0];
		data_.RemoveAt(0);	// not efficient
		return val;
	}
	
	public bool isEmpty() {
		return data_.Count == 0;
	}
}

